# coding: utf-8

"""
Perigon API

The Perigon API provides access to comprehensive news and web content data. To use the API, simply sign up for a Perigon Business Solutions account to obtain your API key. Your available features may vary based on your plan. See the Authentication section for details on how to use your API key.

The version of the OpenAPI document: 1.0.0
Contact: data@perigon.io
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing_extensions import Self

from perigon.models.auth_exception_cause import AuthExceptionCause
from perigon.models.auth_exception_cause_stack_trace_inner import (
    AuthExceptionCauseStackTraceInner,
)
from perigon.models.auth_exception_suppressed_inner import AuthExceptionSuppressedInner


class TooManyRequestsException(BaseModel):
    """
    TooManyRequestsException
    """  # noqa: E501

    cause: Optional[AuthExceptionCause] = None
    stack_trace: Optional[List[AuthExceptionCauseStackTraceInner]] = Field(
        default=None, alias="stackTrace"
    )
    status: Optional[StrictStr] = None
    message: Optional[StrictStr] = None
    suppressed: Optional[List[AuthExceptionSuppressedInner]] = None
    localized_message: Optional[StrictStr] = Field(
        default=None, alias="localizedMessage"
    )
    __properties: ClassVar[List[str]] = [
        "cause",
        "stackTrace",
        "status",
        "message",
        "suppressed",
        "localizedMessage",
    ]

    @field_validator("status")
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(
            [
                "100 CONTINUE",
                "101 SWITCHING_PROTOCOLS",
                "102 PROCESSING",
                "103 CHECKPOINT",
                "200 OK",
                "201 CREATED",
                "202 ACCEPTED",
                "203 NON_AUTHORITATIVE_INFORMATION",
                "204 NO_CONTENT",
                "205 RESET_CONTENT",
                "206 PARTIAL_CONTENT",
                "207 MULTI_STATUS",
                "208 ALREADY_REPORTED",
                "226 IM_USED",
                "300 MULTIPLE_CHOICES",
                "301 MOVED_PERMANENTLY",
                "302 FOUND",
                "302 MOVED_TEMPORARILY",
                "303 SEE_OTHER",
                "304 NOT_MODIFIED",
                "305 USE_PROXY",
                "307 TEMPORARY_REDIRECT",
                "308 PERMANENT_REDIRECT",
                "400 BAD_REQUEST",
                "401 UNAUTHORIZED",
                "402 PAYMENT_REQUIRED",
                "403 FORBIDDEN",
                "404 NOT_FOUND",
                "405 METHOD_NOT_ALLOWED",
                "406 NOT_ACCEPTABLE",
                "407 PROXY_AUTHENTICATION_REQUIRED",
                "408 REQUEST_TIMEOUT",
                "409 CONFLICT",
                "410 GONE",
                "411 LENGTH_REQUIRED",
                "412 PRECONDITION_FAILED",
                "413 PAYLOAD_TOO_LARGE",
                "413 REQUEST_ENTITY_TOO_LARGE",
                "414 URI_TOO_LONG",
                "414 REQUEST_URI_TOO_LONG",
                "415 UNSUPPORTED_MEDIA_TYPE",
                "416 REQUESTED_RANGE_NOT_SATISFIABLE",
                "417 EXPECTATION_FAILED",
                "418 I_AM_A_TEAPOT",
                "419 INSUFFICIENT_SPACE_ON_RESOURCE",
                "420 METHOD_FAILURE",
                "421 DESTINATION_LOCKED",
                "422 UNPROCESSABLE_ENTITY",
                "423 LOCKED",
                "424 FAILED_DEPENDENCY",
                "425 TOO_EARLY",
                "426 UPGRADE_REQUIRED",
                "428 PRECONDITION_REQUIRED",
                "429 TOO_MANY_REQUESTS",
                "431 REQUEST_HEADER_FIELDS_TOO_LARGE",
                "451 UNAVAILABLE_FOR_LEGAL_REASONS",
                "500 INTERNAL_SERVER_ERROR",
                "501 NOT_IMPLEMENTED",
                "502 BAD_GATEWAY",
                "503 SERVICE_UNAVAILABLE",
                "504 GATEWAY_TIMEOUT",
                "505 HTTP_VERSION_NOT_SUPPORTED",
                "506 VARIANT_ALSO_NEGOTIATES",
                "507 INSUFFICIENT_STORAGE",
                "508 LOOP_DETECTED",
                "509 BANDWIDTH_LIMIT_EXCEEDED",
                "510 NOT_EXTENDED",
                "511 NETWORK_AUTHENTICATION_REQUIRED",
            ]
        ):
            raise ValueError(
                "must be one of enum values ('100 CONTINUE', '101 SWITCHING_PROTOCOLS', '102 PROCESSING', '103 CHECKPOINT', '200 OK', '201 CREATED', '202 ACCEPTED', '203 NON_AUTHORITATIVE_INFORMATION', '204 NO_CONTENT', '205 RESET_CONTENT', '206 PARTIAL_CONTENT', '207 MULTI_STATUS', '208 ALREADY_REPORTED', '226 IM_USED', '300 MULTIPLE_CHOICES', '301 MOVED_PERMANENTLY', '302 FOUND', '302 MOVED_TEMPORARILY', '303 SEE_OTHER', '304 NOT_MODIFIED', '305 USE_PROXY', '307 TEMPORARY_REDIRECT', '308 PERMANENT_REDIRECT', '400 BAD_REQUEST', '401 UNAUTHORIZED', '402 PAYMENT_REQUIRED', '403 FORBIDDEN', '404 NOT_FOUND', '405 METHOD_NOT_ALLOWED', '406 NOT_ACCEPTABLE', '407 PROXY_AUTHENTICATION_REQUIRED', '408 REQUEST_TIMEOUT', '409 CONFLICT', '410 GONE', '411 LENGTH_REQUIRED', '412 PRECONDITION_FAILED', '413 PAYLOAD_TOO_LARGE', '413 REQUEST_ENTITY_TOO_LARGE', '414 URI_TOO_LONG', '414 REQUEST_URI_TOO_LONG', '415 UNSUPPORTED_MEDIA_TYPE', '416 REQUESTED_RANGE_NOT_SATISFIABLE', '417 EXPECTATION_FAILED', '418 I_AM_A_TEAPOT', '419 INSUFFICIENT_SPACE_ON_RESOURCE', '420 METHOD_FAILURE', '421 DESTINATION_LOCKED', '422 UNPROCESSABLE_ENTITY', '423 LOCKED', '424 FAILED_DEPENDENCY', '425 TOO_EARLY', '426 UPGRADE_REQUIRED', '428 PRECONDITION_REQUIRED', '429 TOO_MANY_REQUESTS', '431 REQUEST_HEADER_FIELDS_TOO_LARGE', '451 UNAVAILABLE_FOR_LEGAL_REASONS', '500 INTERNAL_SERVER_ERROR', '501 NOT_IMPLEMENTED', '502 BAD_GATEWAY', '503 SERVICE_UNAVAILABLE', '504 GATEWAY_TIMEOUT', '505 HTTP_VERSION_NOT_SUPPORTED', '506 VARIANT_ALSO_NEGOTIATES', '507 INSUFFICIENT_STORAGE', '508 LOOP_DETECTED', '509 BANDWIDTH_LIMIT_EXCEEDED', '510 NOT_EXTENDED', '511 NETWORK_AUTHENTICATION_REQUIRED')"
            )
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TooManyRequestsException from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of cause
        if self.cause:
            _dict["cause"] = self.cause.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in stack_trace (list)
        _items = []
        if self.stack_trace:
            for _item_stack_trace in self.stack_trace:
                if _item_stack_trace:
                    _items.append(_item_stack_trace.to_dict())
            _dict["stackTrace"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in suppressed (list)
        _items = []
        if self.suppressed:
            for _item_suppressed in self.suppressed:
                if _item_suppressed:
                    _items.append(_item_suppressed.to_dict())
            _dict["suppressed"] = _items
        # set to None if cause (nullable) is None
        # and model_fields_set contains the field
        if self.cause is None and "cause" in self.model_fields_set:
            _dict["cause"] = None

        # set to None if stack_trace (nullable) is None
        # and model_fields_set contains the field
        if self.stack_trace is None and "stack_trace" in self.model_fields_set:
            _dict["stackTrace"] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict["status"] = None

        # set to None if message (nullable) is None
        # and model_fields_set contains the field
        if self.message is None and "message" in self.model_fields_set:
            _dict["message"] = None

        # set to None if suppressed (nullable) is None
        # and model_fields_set contains the field
        if self.suppressed is None and "suppressed" in self.model_fields_set:
            _dict["suppressed"] = None

        # set to None if localized_message (nullable) is None
        # and model_fields_set contains the field
        if (
            self.localized_message is None
            and "localized_message" in self.model_fields_set
        ):
            _dict["localizedMessage"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TooManyRequestsException from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "cause": (
                    AuthExceptionCause.from_dict(obj["cause"])
                    if obj.get("cause") is not None
                    else None
                ),
                "stackTrace": (
                    [
                        AuthExceptionCauseStackTraceInner.from_dict(_item)
                        for _item in obj["stackTrace"]
                    ]
                    if obj.get("stackTrace") is not None
                    else None
                ),
                "status": obj.get("status"),
                "message": obj.get("message"),
                "suppressed": (
                    [
                        AuthExceptionSuppressedInner.from_dict(_item)
                        for _item in obj["suppressed"]
                    ]
                    if obj.get("suppressed") is not None
                    else None
                ),
                "localizedMessage": obj.get("localizedMessage"),
            }
        )
        return _obj



# coding: utf-8

"""
Perigon API

The Perigon API provides access to comprehensive news and web content data. To use the API, simply sign up for a Perigon Business Solutions account to obtain your API key. Your available features may vary based on your plan. See the Authentication section for details on how to use your API key.

The version of the OpenAPI document: 1.0.0
Contact: data@perigon.io
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set, Union

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing_extensions import Annotated, Self


class SummaryBody(BaseModel):
    """
    Request body for the Search-Summarizer endpoint. Contains the user’s summary instructions, how many articles to include, which fields to use, the retrieval method, and LLM generation settings.
    """  # noqa: E501

    prompt: Optional[StrictStr] = Field(
        default="You are a helpful assistant tasked with summarizing the search results. In a casual, natural tone, craft a vivid, insightful synthesis in one fluid paragraph (no more than 125 words). Cover as many relevant points as possible, connect ideas, highlight themes, and surface anything interesting or surprising.",
        description="Instructions guiding how the summary should be written. Maximum length: 2 048 characters.",
    )
    max_article_count: Optional[Annotated[int, Field(le=100, strict=True, ge=1)]] = (
        Field(
            default=10,
            description="The maximum number of articles to factor into the summary.",
            alias="maxArticleCount",
        )
    )
    returned_article_count: Optional[
        Annotated[int, Field(le=100, strict=True, ge=1)]
    ] = Field(
        default=10,
        description="The maximum number of articles that should be returned in the response. This can be used to return fewer than maxArticleCount results.",
        alias="returnedArticleCount",
    )
    summarize_fields: Optional[List[StrictStr]] = Field(
        default=None,
        description="Which article fields to include when generating the summary. Up to three values from TITLE, CONTENT, SUMMARY.",
        alias="summarizeFields",
    )
    method: Optional[StrictStr] = Field(
        default="ARTICLES",
        description="Method for selecting articles: ARTICLES (include all matches) or CLUSTERS (one per cluster).",
    )
    model: Optional[StrictStr] = Field(
        default="gpt-4.1", description="The underlying LLM model to use for generation."
    )
    temperature: Optional[
        Union[
            Annotated[float, Field(le=2, strict=True, ge=0)],
            Annotated[int, Field(le=2, strict=True, ge=0)],
        ]
    ] = Field(
        default=0.7,
        description="Sampling temperature for the LLM (0.0 = deterministic to 2.0 = very creative).",
    )
    top_p: Optional[
        Union[
            Annotated[float, Field(le=1, strict=True, ge=0)],
            Annotated[int, Field(le=1, strict=True, ge=0)],
        ]
    ] = Field(
        default=1.0,
        description="Nucleus sampling (top-p) for the LLM (0.0 to 1.0).",
        alias="topP",
    )
    max_tokens: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(
        default=2048,
        description="Maximum number of tokens to generate in the summary.",
        alias="maxTokens",
    )
    __properties: ClassVar[List[str]] = [
        "prompt",
        "maxArticleCount",
        "returnedArticleCount",
        "summarizeFields",
        "method",
        "model",
        "temperature",
        "topP",
        "maxTokens",
    ]

    @field_validator("summarize_fields")
    def summarize_fields_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        for i in value:
            if i not in set(["TITLE", "CONTENT", "SUMMARY"]):
                raise ValueError(
                    "each list item must be one of ('TITLE', 'CONTENT', 'SUMMARY')"
                )
        return value

    @field_validator("method")
    def method_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(["ARTICLES", "CLUSTERS"]):
            raise ValueError("must be one of enum values ('ARTICLES', 'CLUSTERS')")
        return value

    @field_validator("model")
    def model_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(
            [
                "gpt-4o",
                "gpt-4o-mini",
                "gpt-4.1",
                "gpt-4.1-mini",
                "gpt-4.1-nano",
                "llama-3.3-70b-versatile",
                "deepseek-r1-distill-llama-70b",
            ]
        ):
            raise ValueError(
                "must be one of enum values ('gpt-4o', 'gpt-4o-mini', 'gpt-4.1', 'gpt-4.1-mini', 'gpt-4.1-nano', 'llama-3.3-70b-versatile', 'deepseek-r1-distill-llama-70b')"
            )
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SummaryBody from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if prompt (nullable) is None
        # and model_fields_set contains the field
        if self.prompt is None and "prompt" in self.model_fields_set:
            _dict["prompt"] = None

        # set to None if max_article_count (nullable) is None
        # and model_fields_set contains the field
        if (
            self.max_article_count is None
            and "max_article_count" in self.model_fields_set
        ):
            _dict["maxArticleCount"] = None

        # set to None if returned_article_count (nullable) is None
        # and model_fields_set contains the field
        if (
            self.returned_article_count is None
            and "returned_article_count" in self.model_fields_set
        ):
            _dict["returnedArticleCount"] = None

        # set to None if summarize_fields (nullable) is None
        # and model_fields_set contains the field
        if (
            self.summarize_fields is None
            and "summarize_fields" in self.model_fields_set
        ):
            _dict["summarizeFields"] = None

        # set to None if method (nullable) is None
        # and model_fields_set contains the field
        if self.method is None and "method" in self.model_fields_set:
            _dict["method"] = None

        # set to None if model (nullable) is None
        # and model_fields_set contains the field
        if self.model is None and "model" in self.model_fields_set:
            _dict["model"] = None

        # set to None if temperature (nullable) is None
        # and model_fields_set contains the field
        if self.temperature is None and "temperature" in self.model_fields_set:
            _dict["temperature"] = None

        # set to None if top_p (nullable) is None
        # and model_fields_set contains the field
        if self.top_p is None and "top_p" in self.model_fields_set:
            _dict["topP"] = None

        # set to None if max_tokens (nullable) is None
        # and model_fields_set contains the field
        if self.max_tokens is None and "max_tokens" in self.model_fields_set:
            _dict["maxTokens"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SummaryBody from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "prompt": (
                    obj.get("prompt")
                    if obj.get("prompt") is not None
                    else "You are a helpful assistant tasked with summarizing the search results. In a casual, natural tone, craft a vivid, insightful synthesis in one fluid paragraph (no more than 125 words). Cover as many relevant points as possible, connect ideas, highlight themes, and surface anything interesting or surprising."
                ),
                "maxArticleCount": (
                    obj.get("maxArticleCount")
                    if obj.get("maxArticleCount") is not None
                    else 10
                ),
                "returnedArticleCount": (
                    obj.get("returnedArticleCount")
                    if obj.get("returnedArticleCount") is not None
                    else 10
                ),
                "summarizeFields": obj.get("summarizeFields"),
                "method": (
                    obj.get("method") if obj.get("method") is not None else "ARTICLES"
                ),
                "model": (
                    obj.get("model") if obj.get("model") is not None else "gpt-4.1"
                ),
                "temperature": (
                    obj.get("temperature")
                    if obj.get("temperature") is not None
                    else 0.7
                ),
                "topP": obj.get("topP") if obj.get("topP") is not None else 1.0,
                "maxTokens": (
                    obj.get("maxTokens") if obj.get("maxTokens") is not None else 2048
                ),
            }
        )
        return _obj



# coding: utf-8

"""
Perigon API

The Perigon API provides access to comprehensive news and web content data. To use the API, simply sign up for a Perigon Business Solutions account to obtain your API key. Your available features may vary based on your plan. See the Authentication section for details on how to use your API key.

The version of the OpenAPI document: 1.0.0
Contact: data@perigon.io
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, StrictStr
from typing_extensions import Self


class WebResources(BaseModel):
    """
    WebResources
    """  # noqa: E501

    careers: Optional[StrictStr] = None
    about: Optional[StrictStr] = None
    blog: Optional[StrictStr] = None
    events: Optional[StrictStr] = None
    sitemap: Optional[StrictStr] = None
    updates: Optional[StrictStr] = None
    linkedin: Optional[StrictStr] = None
    facebook: Optional[StrictStr] = None
    instagram: Optional[StrictStr] = None
    medium: Optional[StrictStr] = None
    reddit: Optional[StrictStr] = None
    threads: Optional[StrictStr] = None
    tiktok: Optional[StrictStr] = None
    x: Optional[StrictStr] = None
    wellfound: Optional[StrictStr] = None
    youtube: Optional[StrictStr] = None
    wikipedia: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = [
        "careers",
        "about",
        "blog",
        "events",
        "sitemap",
        "updates",
        "linkedin",
        "facebook",
        "instagram",
        "medium",
        "reddit",
        "threads",
        "tiktok",
        "x",
        "wellfound",
        "youtube",
        "wikipedia",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of WebResources from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if careers (nullable) is None
        # and model_fields_set contains the field
        if self.careers is None and "careers" in self.model_fields_set:
            _dict["careers"] = None

        # set to None if about (nullable) is None
        # and model_fields_set contains the field
        if self.about is None and "about" in self.model_fields_set:
            _dict["about"] = None

        # set to None if blog (nullable) is None
        # and model_fields_set contains the field
        if self.blog is None and "blog" in self.model_fields_set:
            _dict["blog"] = None

        # set to None if events (nullable) is None
        # and model_fields_set contains the field
        if self.events is None and "events" in self.model_fields_set:
            _dict["events"] = None

        # set to None if sitemap (nullable) is None
        # and model_fields_set contains the field
        if self.sitemap is None and "sitemap" in self.model_fields_set:
            _dict["sitemap"] = None

        # set to None if updates (nullable) is None
        # and model_fields_set contains the field
        if self.updates is None and "updates" in self.model_fields_set:
            _dict["updates"] = None

        # set to None if linkedin (nullable) is None
        # and model_fields_set contains the field
        if self.linkedin is None and "linkedin" in self.model_fields_set:
            _dict["linkedin"] = None

        # set to None if facebook (nullable) is None
        # and model_fields_set contains the field
        if self.facebook is None and "facebook" in self.model_fields_set:
            _dict["facebook"] = None

        # set to None if instagram (nullable) is None
        # and model_fields_set contains the field
        if self.instagram is None and "instagram" in self.model_fields_set:
            _dict["instagram"] = None

        # set to None if medium (nullable) is None
        # and model_fields_set contains the field
        if self.medium is None and "medium" in self.model_fields_set:
            _dict["medium"] = None

        # set to None if reddit (nullable) is None
        # and model_fields_set contains the field
        if self.reddit is None and "reddit" in self.model_fields_set:
            _dict["reddit"] = None

        # set to None if threads (nullable) is None
        # and model_fields_set contains the field
        if self.threads is None and "threads" in self.model_fields_set:
            _dict["threads"] = None

        # set to None if tiktok (nullable) is None
        # and model_fields_set contains the field
        if self.tiktok is None and "tiktok" in self.model_fields_set:
            _dict["tiktok"] = None

        # set to None if x (nullable) is None
        # and model_fields_set contains the field
        if self.x is None and "x" in self.model_fields_set:
            _dict["x"] = None

        # set to None if wellfound (nullable) is None
        # and model_fields_set contains the field
        if self.wellfound is None and "wellfound" in self.model_fields_set:
            _dict["wellfound"] = None

        # set to None if youtube (nullable) is None
        # and model_fields_set contains the field
        if self.youtube is None and "youtube" in self.model_fields_set:
            _dict["youtube"] = None

        # set to None if wikipedia (nullable) is None
        # and model_fields_set contains the field
        if self.wikipedia is None and "wikipedia" in self.model_fields_set:
            _dict["wikipedia"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of WebResources from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "careers": obj.get("careers"),
                "about": obj.get("about"),
                "blog": obj.get("blog"),
                "events": obj.get("events"),
                "sitemap": obj.get("sitemap"),
                "updates": obj.get("updates"),
                "linkedin": obj.get("linkedin"),
                "facebook": obj.get("facebook"),
                "instagram": obj.get("instagram"),
                "medium": obj.get("medium"),
                "reddit": obj.get("reddit"),
                "threads": obj.get("threads"),
                "tiktok": obj.get("tiktok"),
                "x": obj.get("x"),
                "wellfound": obj.get("wellfound"),
                "youtube": obj.get("youtube"),
                "wikipedia": obj.get("wikipedia"),
            }
        )
        return _obj



# coding: utf-8

"""
Perigon API

The Perigon API provides access to comprehensive news and web content data. To use the API, simply sign up for a Perigon Business Solutions account to obtain your API key. Your available features may vary based on your plan. See the Authentication section for details on how to use your API key.

The version of the OpenAPI document: 1.0.0
Contact: data@perigon.io
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from datetime import datetime
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictBool
from typing_extensions import Annotated, Self

from perigon.models.article_search_filter import ArticleSearchFilter


class ArticleSearchParams(BaseModel):
    """
    ArticleSearchParams
    """  # noqa: E501

    prompt: Annotated[str, Field(min_length=0, strict=True, max_length=1024)] = Field(
        description="Natural language query to search the news articles database"
    )
    filter: Optional[ArticleSearchFilter] = None
    pub_date_from: Optional[datetime] = Field(
        default=None,
        description="'pubDateFrom' filter, will search articles published after the specified date, the date could be passed as ISO or 'yyyy-mm-dd'. Date time in ISO format, ie. 2024-01-01T00:00:00 - Default: Only articles with a pubDate within the last 30 days of the request",
        alias="pubDateFrom",
    )
    pub_date_to: Optional[datetime] = Field(
        default=None,
        description="'pubDateTo' filter, will search articles published before the specified date, the date could be passed as ISO or 'yyyy-mm-dd'. Date time in ISO format, ie. 2024-01-01T00:00:00",
        alias="pubDateTo",
    )
    show_reprints: Optional[StrictBool] = Field(
        default=True,
        description="Whether to return reprints in the response or not. Reprints are usually wired articles from sources like AP or Reuters that are reprinted in multiple sources at the same time. By default, this parameter is 'true'.",
        alias="showReprints",
    )
    size: Optional[Annotated[int, Field(le=100, strict=True, ge=1)]] = Field(
        default=10, description="The number of items per page."
    )
    page: Optional[Annotated[int, Field(le=10000, strict=True, ge=0)]] = Field(
        default=0, description="The page number to retrieve."
    )
    __properties: ClassVar[List[str]] = [
        "prompt",
        "filter",
        "pubDateFrom",
        "pubDateTo",
        "showReprints",
        "size",
        "page",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ArticleSearchParams from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of filter
        if self.filter:
            _dict["filter"] = self.filter.to_dict()
        # set to None if filter (nullable) is None
        # and model_fields_set contains the field
        if self.filter is None and "filter" in self.model_fields_set:
            _dict["filter"] = None

        # set to None if pub_date_from (nullable) is None
        # and model_fields_set contains the field
        if self.pub_date_from is None and "pub_date_from" in self.model_fields_set:
            _dict["pubDateFrom"] = None

        # set to None if pub_date_to (nullable) is None
        # and model_fields_set contains the field
        if self.pub_date_to is None and "pub_date_to" in self.model_fields_set:
            _dict["pubDateTo"] = None

        # set to None if show_reprints (nullable) is None
        # and model_fields_set contains the field
        if self.show_reprints is None and "show_reprints" in self.model_fields_set:
            _dict["showReprints"] = None

        # set to None if size (nullable) is None
        # and model_fields_set contains the field
        if self.size is None and "size" in self.model_fields_set:
            _dict["size"] = None

        # set to None if page (nullable) is None
        # and model_fields_set contains the field
        if self.page is None and "page" in self.model_fields_set:
            _dict["page"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ArticleSearchParams from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "prompt": obj.get("prompt"),
                "filter": (
                    ArticleSearchFilter.from_dict(obj["filter"])
                    if obj.get("filter") is not None
                    else None
                ),
                "pubDateFrom": obj.get("pubDateFrom"),
                "pubDateTo": obj.get("pubDateTo"),
                "showReprints": (
                    obj.get("showReprints")
                    if obj.get("showReprints") is not None
                    else True
                ),
                "size": obj.get("size") if obj.get("size") is not None else 10,
                "page": obj.get("page") if obj.get("page") is not None else 0,
            }
        )
        return _obj



# coding: utf-8

"""
Perigon API

The Perigon API provides access to comprehensive news and web content data. To use the API, simply sign up for a Perigon Business Solutions account to obtain your API key. Your available features may vary based on your plan. See the Authentication section for details on how to use your API key.

The version of the OpenAPI document: 1.0.0
Contact: data@perigon.io
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing_extensions import Self


class AuthExceptionCauseStackTraceInner(BaseModel):
    """
    AuthExceptionCauseStackTraceInner
    """  # noqa: E501

    class_loader_name: Optional[StrictStr] = Field(
        default=None, alias="classLoaderName"
    )
    module_name: Optional[StrictStr] = Field(default=None, alias="moduleName")
    module_version: Optional[StrictStr] = Field(default=None, alias="moduleVersion")
    method_name: Optional[StrictStr] = Field(default=None, alias="methodName")
    file_name: Optional[StrictStr] = Field(default=None, alias="fileName")
    line_number: Optional[StrictInt] = Field(default=None, alias="lineNumber")
    native_method: Optional[StrictBool] = Field(default=None, alias="nativeMethod")
    class_name: Optional[StrictStr] = Field(default=None, alias="className")
    __properties: ClassVar[List[str]] = [
        "classLoaderName",
        "moduleName",
        "moduleVersion",
        "methodName",
        "fileName",
        "lineNumber",
        "nativeMethod",
        "className",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of AuthExceptionCauseStackTraceInner from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AuthExceptionCauseStackTraceInner from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "classLoaderName": obj.get("classLoaderName"),
                "moduleName": obj.get("moduleName"),
                "moduleVersion": obj.get("moduleVersion"),
                "methodName": obj.get("methodName"),
                "fileName": obj.get("fileName"),
                "lineNumber": obj.get("lineNumber"),
                "nativeMethod": obj.get("nativeMethod"),
                "className": obj.get("className"),
            }
        )
        return _obj



# coding: utf-8

"""
Perigon API

The Perigon API provides access to comprehensive news and web content data. To use the API, simply sign up for a Perigon Business Solutions account to obtain your API key. Your available features may vary based on your plan. See the Authentication section for details on how to use your API key.

The version of the OpenAPI document: 1.0.0
Contact: data@perigon.io
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing_extensions import Self

from perigon.models.key_point import KeyPoint
from perigon.models.question import Question


class StoryHistoryRecord(BaseModel):
    """
    StoryHistoryRecord
    """  # noqa: E501

    cluster_id: Optional[StrictStr] = Field(default=None, alias="clusterId")
    created_at: Optional[StrictStr] = Field(default=None, alias="createdAt")
    name: Optional[StrictStr] = None
    triggered_at: Optional[StrictStr] = Field(default=None, alias="triggeredAt")
    summary: Optional[StrictStr] = None
    short_summary: Optional[StrictStr] = Field(default=None, alias="shortSummary")
    changelog: Optional[StrictStr] = None
    key_points: Optional[List[KeyPoint]] = Field(default=None, alias="keyPoints")
    questions: Optional[List[Question]] = None
    __properties: ClassVar[List[str]] = [
        "clusterId",
        "createdAt",
        "name",
        "triggeredAt",
        "summary",
        "shortSummary",
        "changelog",
        "keyPoints",
        "questions",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of StoryHistoryRecord from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in key_points (list)
        _items = []
        if self.key_points:
            for _item_key_points in self.key_points:
                if _item_key_points:
                    _items.append(_item_key_points.to_dict())
            _dict["keyPoints"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in questions (list)
        _items = []
        if self.questions:
            for _item_questions in self.questions:
                if _item_questions:
                    _items.append(_item_questions.to_dict())
            _dict["questions"] = _items
        # set to None if cluster_id (nullable) is None
        # and model_fields_set contains the field
        if self.cluster_id is None and "cluster_id" in self.model_fields_set:
            _dict["clusterId"] = None

        # set to None if created_at (nullable) is None
        # and model_fields_set contains the field
        if self.created_at is None and "created_at" in self.model_fields_set:
            _dict["createdAt"] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict["name"] = None

        # set to None if triggered_at (nullable) is None
        # and model_fields_set contains the field
        if self.triggered_at is None and "triggered_at" in self.model_fields_set:
            _dict["triggeredAt"] = None

        # set to None if summary (nullable) is None
        # and model_fields_set contains the field
        if self.summary is None and "summary" in self.model_fields_set:
            _dict["summary"] = None

        # set to None if short_summary (nullable) is None
        # and model_fields_set contains the field
        if self.short_summary is None and "short_summary" in self.model_fields_set:
            _dict["shortSummary"] = None

        # set to None if changelog (nullable) is None
        # and model_fields_set contains the field
        if self.changelog is None and "changelog" in self.model_fields_set:
            _dict["changelog"] = None

        # set to None if key_points (nullable) is None
        # and model_fields_set contains the field
        if self.key_points is None and "key_points" in self.model_fields_set:
            _dict["keyPoints"] = None

        # set to None if questions (nullable) is None
        # and model_fields_set contains the field
        if self.questions is None and "questions" in self.model_fields_set:
            _dict["questions"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of StoryHistoryRecord from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "clusterId": obj.get("clusterId"),
                "createdAt": obj.get("createdAt"),
                "name": obj.get("name"),
                "triggeredAt": obj.get("triggeredAt"),
                "summary": obj.get("summary"),
                "shortSummary": obj.get("shortSummary"),
                "changelog": obj.get("changelog"),
                "keyPoints": (
                    [KeyPoint.from_dict(_item) for _item in obj["keyPoints"]]
                    if obj.get("keyPoints") is not None
                    else None
                ),
                "questions": (
                    [Question.from_dict(_item) for _item in obj["questions"]]
                    if obj.get("questions") is not None
                    else None
                ),
            }
        )
        return _obj



# coding: utf-8

"""
Perigon API

The Perigon API provides access to comprehensive news and web content data. To use the API, simply sign up for a Perigon Business Solutions account to obtain your API key. Your available features may vary based on your plan. See the Authentication section for details on how to use your API key.

The version of the OpenAPI document: 1.0.0
Contact: data@perigon.io
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from datetime import datetime
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictInt
from typing_extensions import Annotated, Self

from perigon.models.wikipedia_search_filter import WikipediaSearchFilter


class WikipediaSearchParams(BaseModel):
    """
    WikipediaSearchParams
    """  # noqa: E501

    prompt: Annotated[str, Field(min_length=0, strict=True, max_length=1024)] = Field(
        description="Natural language query to search the Wikipedia pages database"
    )
    filter: Optional[WikipediaSearchFilter] = None
    wiki_revision_from: Optional[datetime] = Field(
        default=None,
        description="'wikiRevisionFrom' filter, will search pages modified after the specified date, the date could be passed as ISO or 'yyyy-mm-dd'. Date time in ISO format, ie. 2024-01-01T00:00:00.",
        alias="wikiRevisionFrom",
    )
    wiki_revision_to: Optional[datetime] = Field(
        default=None,
        description="'wikiRevisionFrom' filter, will search pages modified before the specified date, the date could be passed as ISO or 'yyyy-mm-dd'. Date time in ISO format, ie. 2024-01-01T00:00:00.",
        alias="wikiRevisionTo",
    )
    pageviews_from: Optional[StrictInt] = Field(
        default=None,
        description="'pageviewsFrom' filter, will search pages with at least the provided number of views per day.",
        alias="pageviewsFrom",
    )
    pageviews_to: Optional[StrictInt] = Field(
        default=None,
        description="'pageviewsFrom' filter, will search pages with at most the provided number of views per day.",
        alias="pageviewsTo",
    )
    size: Optional[Annotated[int, Field(le=100, strict=True, ge=1)]] = Field(
        default=10, description="The number of items per page."
    )
    page: Optional[Annotated[int, Field(le=10000, strict=True, ge=0)]] = Field(
        default=0, description="The page number to retrieve."
    )
    __properties: ClassVar[List[str]] = [
        "prompt",
        "filter",
        "wikiRevisionFrom",
        "wikiRevisionTo",
        "pageviewsFrom",
        "pageviewsTo",
        "size",
        "page",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of WikipediaSearchParams from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of filter
        if self.filter:
            _dict["filter"] = self.filter.to_dict()
        # set to None if filter (nullable) is None
        # and model_fields_set contains the field
        if self.filter is None and "filter" in self.model_fields_set:
            _dict["filter"] = None

        # set to None if wiki_revision_from (nullable) is None
        # and model_fields_set contains the field
        if (
            self.wiki_revision_from is None
            and "wiki_revision_from" in self.model_fields_set
        ):
            _dict["wikiRevisionFrom"] = None

        # set to None if wiki_revision_to (nullable) is None
        # and model_fields_set contains the field
        if (
            self.wiki_revision_to is None
            and "wiki_revision_to" in self.model_fields_set
        ):
            _dict["wikiRevisionTo"] = None

        # set to None if pageviews_from (nullable) is None
        # and model_fields_set contains the field
        if self.pageviews_from is None and "pageviews_from" in self.model_fields_set:
            _dict["pageviewsFrom"] = None

        # set to None if pageviews_to (nullable) is None
        # and model_fields_set contains the field
        if self.pageviews_to is None and "pageviews_to" in self.model_fields_set:
            _dict["pageviewsTo"] = None

        # set to None if size (nullable) is None
        # and model_fields_set contains the field
        if self.size is None and "size" in self.model_fields_set:
            _dict["size"] = None

        # set to None if page (nullable) is None
        # and model_fields_set contains the field
        if self.page is None and "page" in self.model_fields_set:
            _dict["page"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of WikipediaSearchParams from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "prompt": obj.get("prompt"),
                "filter": (
                    WikipediaSearchFilter.from_dict(obj["filter"])
                    if obj.get("filter") is not None
                    else None
                ),
                "wikiRevisionFrom": obj.get("wikiRevisionFrom"),
                "wikiRevisionTo": obj.get("wikiRevisionTo"),
                "pageviewsFrom": obj.get("pageviewsFrom"),
                "pageviewsTo": obj.get("pageviewsTo"),
                "size": obj.get("size") if obj.get("size") is not None else 10,
                "page": obj.get("page") if obj.get("page") is not None else 0,
            }
        )
        return _obj



# coding: utf-8

# flake8: noqa
"""
Perigon API

The Perigon API provides access to comprehensive news and web content data. To use the API, simply sign up for a Perigon Business Solutions account to obtain your API key. Your available features may vary based on your plan. See the Authentication section for details on how to use your API key.

The version of the OpenAPI document: 1.0.0
Contact: data@perigon.io
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from perigon.models.all_endpoint_sort_by import AllEndpointSortBy
from perigon.models.article import Article
from perigon.models.article_search_filter import ArticleSearchFilter
from perigon.models.article_search_params import ArticleSearchParams
from perigon.models.articles_vector_search_result import ArticlesVectorSearchResult
from perigon.models.category_holder import CategoryHolder
from perigon.models.category_with_score_holder import CategoryWithScoreHolder
from perigon.models.company import Company
from perigon.models.company_count import CompanyCount
from perigon.models.company_holder import CompanyHolder
from perigon.models.company_search_result import CompanySearchResult
from perigon.models.coordinate import Coordinate
from perigon.models.coordinate_filter import CoordinateFilter
from perigon.models.entity_holder import EntityHolder
from perigon.models.error_response import ErrorResponse
from perigon.models.event_type_holder import EventTypeHolder
from perigon.models.id_name_holder import IdNameHolder
from perigon.models.image_holder import ImageHolder
from perigon.models.journalist import Journalist
from perigon.models.journalist_search_result import JournalistSearchResult
from perigon.models.key_point import KeyPoint
from perigon.models.keyword_holder import KeywordHolder
from perigon.models.label_holder import LabelHolder
from perigon.models.location_count import LocationCount
from perigon.models.location_holder import LocationHolder
from perigon.models.name_count import NameCount
from perigon.models.news_cluster import NewsCluster
from perigon.models.people_search_result import PeopleSearchResult
from perigon.models.person import Person
from perigon.models.person_count import PersonCount
from perigon.models.person_holder import PersonHolder
from perigon.models.place import Place
from perigon.models.query_search_result import QuerySearchResult
from perigon.models.question import Question
from perigon.models.record_stat_holder import RecordStatHolder
from perigon.models.scored_data_article import ScoredDataArticle
from perigon.models.scored_data_wiki_data import ScoredDataWikiData
from perigon.models.sentiment_holder import SentimentHolder
from perigon.models.sort_by import SortBy
from perigon.models.source import Source
from perigon.models.source_holder import SourceHolder
from perigon.models.source_location import SourceLocation
from perigon.models.source_search_result import SourceSearchResult
from perigon.models.source_top_stat_holder import SourceTopStatHolder
from perigon.models.stat_result import StatResult
from perigon.models.story_history_record import StoryHistoryRecord
from perigon.models.story_history_result import StoryHistoryResult
from perigon.models.story_search_result import StorySearchResult
from perigon.models.summary_body import SummaryBody
from perigon.models.summary_search_result import SummarySearchResult
from perigon.models.symbol_holder import SymbolHolder
from perigon.models.topic_dto import TopicDto
from perigon.models.topic_holder import TopicHolder
from perigon.models.topic_labels import TopicLabels
from perigon.models.topic_search_result import TopicSearchResult
from perigon.models.web_resources import WebResources
from perigon.models.wiki_data import WikiData
from perigon.models.wiki_page import WikiPage
from perigon.models.wiki_page_section_holder import WikiPageSectionHolder
from perigon.models.wikidata_date_holder import WikidataDateHolder
from perigon.models.wikidata_label_holder import WikidataLabelHolder
from perigon.models.wikidata_political_party_holder import WikidataPoliticalPartyHolder
from perigon.models.wikidata_position_holder import WikidataPositionHolder
from perigon.models.wikipedia_search_filter import WikipediaSearchFilter
from perigon.models.wikipedia_search_params import WikipediaSearchParams
from perigon.models.wikipedia_search_result import WikipediaSearchResult
from perigon.models.wikipedia_vector_search_result import WikipediaVectorSearchResult



# coding: utf-8

"""
Perigon API

The Perigon API provides access to comprehensive news and web content data. To use the API, simply sign up for a Perigon Business Solutions account to obtain your API key. Your available features may vary based on your plan. See the Authentication section for details on how to use your API key.

The version of the OpenAPI document: 1.0.0
Contact: data@perigon.io
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing_extensions import Self

from perigon.models.wiki_page_section_holder import WikiPageSectionHolder
from perigon.models.wikidata_label_holder import WikidataLabelHolder


class WikiPage(BaseModel):
    """
    WikiPage
    """  # noqa: E501

    id: Optional[StrictStr] = None
    scraped_at: Optional[StrictStr] = Field(default=None, alias="scrapedAt")
    wiki_page_id: Optional[StrictInt] = Field(default=None, alias="wikiPageId")
    wiki_revision_id: Optional[StrictInt] = Field(default=None, alias="wikiRevisionId")
    wiki_revision_ts: Optional[StrictStr] = Field(default=None, alias="wikiRevisionTs")
    wiki_code: Optional[StrictStr] = Field(default=None, alias="wikiCode")
    wiki_namespace: Optional[StrictInt] = Field(default=None, alias="wikiNamespace")
    wiki_title: Optional[StrictStr] = Field(default=None, alias="wikiTitle")
    url: Optional[StrictStr] = None
    top_image: Optional[StrictStr] = Field(default=None, alias="topImage")
    wikidata_id: Optional[StrictStr] = Field(default=None, alias="wikidataId")
    wikidata_instance_of: Optional[List[WikidataLabelHolder]] = Field(
        default=None, alias="wikidataInstanceOf"
    )
    redirect_titles: Optional[List[StrictStr]] = Field(
        default=None, alias="redirectTitles"
    )
    summary: Optional[StrictStr] = None
    sections: Optional[List[WikiPageSectionHolder]] = None
    categories: Optional[List[StrictStr]] = None
    external_links: Optional[List[StrictStr]] = Field(
        default=None, alias="externalLinks"
    )
    references: Optional[List[StrictStr]] = None
    pageviews: Optional[StrictInt] = None
    __properties: ClassVar[List[str]] = [
        "id",
        "scrapedAt",
        "wikiPageId",
        "wikiRevisionId",
        "wikiRevisionTs",
        "wikiCode",
        "wikiNamespace",
        "wikiTitle",
        "url",
        "topImage",
        "wikidataId",
        "wikidataInstanceOf",
        "redirectTitles",
        "summary",
        "sections",
        "categories",
        "externalLinks",
        "references",
        "pageviews",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of WikiPage from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in wikidata_instance_of (list)
        _items = []
        if self.wikidata_instance_of:
            for _item_wikidata_instance_of in self.wikidata_instance_of:
                if _item_wikidata_instance_of:
                    _items.append(_item_wikidata_instance_of.to_dict())
            _dict["wikidataInstanceOf"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in sections (list)
        _items = []
        if self.sections:
            for _item_sections in self.sections:
                if _item_sections:
                    _items.append(_item_sections.to_dict())
            _dict["sections"] = _items
        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict["id"] = None

        # set to None if scraped_at (nullable) is None
        # and model_fields_set contains the field
        if self.scraped_at is None and "scraped_at" in self.model_fields_set:
            _dict["scrapedAt"] = None

        # set to None if wiki_page_id (nullable) is None
        # and model_fields_set contains the field
        if self.wiki_page_id is None and "wiki_page_id" in self.model_fields_set:
            _dict["wikiPageId"] = None

        # set to None if wiki_revision_id (nullable) is None
        # and model_fields_set contains the field
        if (
            self.wiki_revision_id is None
            and "wiki_revision_id" in self.model_fields_set
        ):
            _dict["wikiRevisionId"] = None

        # set to None if wiki_revision_ts (nullable) is None
        # and model_fields_set contains the field
        if (
            self.wiki_revision_ts is None
            and "wiki_revision_ts" in self.model_fields_set
        ):
            _dict["wikiRevisionTs"] = None

        # set to None if wiki_code (nullable) is None
        # and model_fields_set contains the field
        if self.wiki_code is None and "wiki_code" in self.model_fields_set:
            _dict["wikiCode"] = None

        # set to None if wiki_namespace (nullable) is None
        # and model_fields_set contains the field
        if self.wiki_namespace is None and "wiki_namespace" in self.model_fields_set:
            _dict["wikiNamespace"] = None

        # set to None if wiki_title (nullable) is None
        # and model_fields_set contains the field
        if self.wiki_title is None and "wiki_title" in self.model_fields_set:
            _dict["wikiTitle"] = None

        # set to None if url (nullable) is None
        # and model_fields_set contains the field
        if self.url is None and "url" in self.model_fields_set:
            _dict["url"] = None

        # set to None if top_image (nullable) is None
        # and model_fields_set contains the field
        if self.top_image is None and "top_image" in self.model_fields_set:
            _dict["topImage"] = None

        # set to None if wikidata_id (nullable) is None
        # and model_fields_set contains the field
        if self.wikidata_id is None and "wikidata_id" in self.model_fields_set:
            _dict["wikidataId"] = None

        # set to None if wikidata_instance_of (nullable) is None
        # and model_fields_set contains the field
        if (
            self.wikidata_instance_of is None
            and "wikidata_instance_of" in self.model_fields_set
        ):
            _dict["wikidataInstanceOf"] = None

        # set to None if redirect_titles (nullable) is None
        # and model_fields_set contains the field
        if self.redirect_titles is None and "redirect_titles" in self.model_fields_set:
            _dict["redirectTitles"] = None

        # set to None if summary (nullable) is None
        # and model_fields_set contains the field
        if self.summary is None and "summary" in self.model_fields_set:
            _dict["summary"] = None

        # set to None if sections (nullable) is None
        # and model_fields_set contains the field
        if self.sections is None and "sections" in self.model_fields_set:
            _dict["sections"] = None

        # set to None if categories (nullable) is None
        # and model_fields_set contains the field
        if self.categories is None and "categories" in self.model_fields_set:
            _dict["categories"] = None

        # set to None if external_links (nullable) is None
        # and model_fields_set contains the field
        if self.external_links is None and "external_links" in self.model_fields_set:
            _dict["externalLinks"] = None

        # set to None if references (nullable) is None
        # and model_fields_set contains the field
        if self.references is None and "references" in self.model_fields_set:
            _dict["references"] = None

        # set to None if pageviews (nullable) is None
        # and model_fields_set contains the field
        if self.pageviews is None and "pageviews" in self.model_fields_set:
            _dict["pageviews"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of WikiPage from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "scrapedAt": obj.get("scrapedAt"),
                "wikiPageId": obj.get("wikiPageId"),
                "wikiRevisionId": obj.get("wikiRevisionId"),
                "wikiRevisionTs": obj.get("wikiRevisionTs"),
                "wikiCode": obj.get("wikiCode"),
                "wikiNamespace": obj.get("wikiNamespace"),
                "wikiTitle": obj.get("wikiTitle"),
                "url": obj.get("url"),
                "topImage": obj.get("topImage"),
                "wikidataId": obj.get("wikidataId"),
                "wikidataInstanceOf": (
                    [
                        WikidataLabelHolder.from_dict(_item)
                        for _item in obj["wikidataInstanceOf"]
                    ]
                    if obj.get("wikidataInstanceOf") is not None
                    else None
                ),
                "redirectTitles": obj.get("redirectTitles"),
                "summary": obj.get("summary"),
                "sections": (
                    [
                        WikiPageSectionHolder.from_dict(_item)
                        for _item in obj["sections"]
                    ]
                    if obj.get("sections") is not None
                    else None
                ),
                "categories": obj.get("categories"),
                "externalLinks": obj.get("externalLinks"),
                "references": obj.get("references"),
                "pageviews": obj.get("pageviews"),
            }
        )
        return _obj



# coding: utf-8

"""
Perigon API

The Perigon API provides access to comprehensive news and web content data. To use the API, simply sign up for a Perigon Business Solutions account to obtain your API key. Your available features may vary based on your plan. See the Authentication section for details on how to use your API key.

The version of the OpenAPI document: 1.0.0
Contact: data@perigon.io
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing_extensions import Self

from perigon.models.image_holder import ImageHolder
from perigon.models.wikidata_date_holder import WikidataDateHolder
from perigon.models.wikidata_label_holder import WikidataLabelHolder
from perigon.models.wikidata_political_party_holder import WikidataPoliticalPartyHolder
from perigon.models.wikidata_position_holder import WikidataPositionHolder


class Person(BaseModel):
    """
    Person
    """  # noqa: E501

    wikidata_id: Optional[StrictStr] = Field(default=None, alias="wikidataId")
    created_at: Optional[StrictStr] = Field(default=None, alias="createdAt")
    updated_at: Optional[StrictStr] = Field(default=None, alias="updatedAt")
    name: Optional[StrictStr] = None
    gender: Optional[WikidataLabelHolder] = None
    date_of_birth: Optional[WikidataDateHolder] = Field(
        default=None, alias="dateOfBirth"
    )
    date_of_death: Optional[WikidataDateHolder] = Field(
        default=None, alias="dateOfDeath"
    )
    description: Optional[StrictStr] = None
    aliases: Optional[List[StrictStr]] = None
    occupation: Optional[List[WikidataLabelHolder]] = None
    position: Optional[List[WikidataPositionHolder]] = None
    political_party: Optional[List[WikidataPoliticalPartyHolder]] = Field(
        default=None, alias="politicalParty"
    )
    image: Optional[ImageHolder] = None
    abstract: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = [
        "wikidataId",
        "createdAt",
        "updatedAt",
        "name",
        "gender",
        "dateOfBirth",
        "dateOfDeath",
        "description",
        "aliases",
        "occupation",
        "position",
        "politicalParty",
        "image",
        "abstract",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Person from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of gender
        if self.gender:
            _dict["gender"] = self.gender.to_dict()
        # override the default output from pydantic by calling `to_dict()` of date_of_birth
        if self.date_of_birth:
            _dict["dateOfBirth"] = self.date_of_birth.to_dict()
        # override the default output from pydantic by calling `to_dict()` of date_of_death
        if self.date_of_death:
            _dict["dateOfDeath"] = self.date_of_death.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in occupation (list)
        _items = []
        if self.occupation:
            for _item_occupation in self.occupation:
                if _item_occupation:
                    _items.append(_item_occupation.to_dict())
            _dict["occupation"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in position (list)
        _items = []
        if self.position:
            for _item_position in self.position:
                if _item_position:
                    _items.append(_item_position.to_dict())
            _dict["position"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in political_party (list)
        _items = []
        if self.political_party:
            for _item_political_party in self.political_party:
                if _item_political_party:
                    _items.append(_item_political_party.to_dict())
            _dict["politicalParty"] = _items
        # override the default output from pydantic by calling `to_dict()` of image
        if self.image:
            _dict["image"] = self.image.to_dict()
        # set to None if wikidata_id (nullable) is None
        # and model_fields_set contains the field
        if self.wikidata_id is None and "wikidata_id" in self.model_fields_set:
            _dict["wikidataId"] = None

        # set to None if created_at (nullable) is None
        # and model_fields_set contains the field
        if self.created_at is None and "created_at" in self.model_fields_set:
            _dict["createdAt"] = None

        # set to None if updated_at (nullable) is None
        # and model_fields_set contains the field
        if self.updated_at is None and "updated_at" in self.model_fields_set:
            _dict["updatedAt"] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict["name"] = None

        # set to None if gender (nullable) is None
        # and model_fields_set contains the field
        if self.gender is None and "gender" in self.model_fields_set:
            _dict["gender"] = None

        # set to None if date_of_birth (nullable) is None
        # and model_fields_set contains the field
        if self.date_of_birth is None and "date_of_birth" in self.model_fields_set:
            _dict["dateOfBirth"] = None

        # set to None if date_of_death (nullable) is None
        # and model_fields_set contains the field
        if self.date_of_death is None and "date_of_death" in self.model_fields_set:
            _dict["dateOfDeath"] = None

        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict["description"] = None

        # set to None if aliases (nullable) is None
        # and model_fields_set contains the field
        if self.aliases is None and "aliases" in self.model_fields_set:
            _dict["aliases"] = None

        # set to None if occupation (nullable) is None
        # and model_fields_set contains the field
        if self.occupation is None and "occupation" in self.model_fields_set:
            _dict["occupation"] = None

        # set to None if position (nullable) is None
        # and model_fields_set contains the field
        if self.position is None and "position" in self.model_fields_set:
            _dict["position"] = None

        # set to None if political_party (nullable) is None
        # and model_fields_set contains the field
        if self.political_party is None and "political_party" in self.model_fields_set:
            _dict["politicalParty"] = None

        # set to None if image (nullable) is None
        # and model_fields_set contains the field
        if self.image is None and "image" in self.model_fields_set:
            _dict["image"] = None

        # set to None if abstract (nullable) is None
        # and model_fields_set contains the field
        if self.abstract is None and "abstract" in self.model_fields_set:
            _dict["abstract"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Person from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "wikidataId": obj.get("wikidataId"),
                "createdAt": obj.get("createdAt"),
                "updatedAt": obj.get("updatedAt"),
                "name": obj.get("name"),
                "gender": (
                    WikidataLabelHolder.from_dict(obj["gender"])
                    if obj.get("gender") is not None
                    else None
                ),
                "dateOfBirth": (
                    WikidataDateHolder.from_dict(obj["dateOfBirth"])
                    if obj.get("dateOfBirth") is not None
                    else None
                ),
                "dateOfDeath": (
                    WikidataDateHolder.from_dict(obj["dateOfDeath"])
                    if obj.get("dateOfDeath") is not None
                    else None
                ),
                "description": obj.get("description"),
                "aliases": obj.get("aliases"),
                "occupation": (
                    [
                        WikidataLabelHolder.from_dict(_item)
                        for _item in obj["occupation"]
                    ]
                    if obj.get("occupation") is not None
                    else None
                ),
                "position": (
                    [
                        WikidataPositionHolder.from_dict(_item)
                        for _item in obj["position"]
                    ]
                    if obj.get("position") is not None
                    else None
                ),
                "politicalParty": (
                    [
                        WikidataPoliticalPartyHolder.from_dict(_item)
                        for _item in obj["politicalParty"]
                    ]
                    if obj.get("politicalParty") is not None
                    else None
                ),
                "image": (
                    ImageHolder.from_dict(obj["image"])
                    if obj.get("image") is not None
                    else None
                ),
                "abstract": obj.get("abstract"),
            }
        )
        return _obj



# coding: utf-8

"""
Perigon API

The Perigon API provides access to comprehensive news and web content data. To use the API, simply sign up for a Perigon Business Solutions account to obtain your API key. Your available features may vary based on your plan. See the Authentication section for details on how to use your API key.

The version of the OpenAPI document: 1.0.0
Contact: data@perigon.io
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictInt
from typing_extensions import Self

from perigon.models.journalist import Journalist


class JournalistSearchResult(BaseModel):
    """
    Journalist search result
    """  # noqa: E501

    status: StrictInt
    num_results: StrictInt = Field(alias="numResults")
    results: List[Journalist]
    __properties: ClassVar[List[str]] = ["status", "numResults", "results"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of JournalistSearchResult from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in results (list)
        _items = []
        if self.results:
            for _item_results in self.results:
                if _item_results:
                    _items.append(_item_results.to_dict())
            _dict["results"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of JournalistSearchResult from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "status": obj.get("status"),
                "numResults": obj.get("numResults"),
                "results": (
                    [Journalist.from_dict(_item) for _item in obj["results"]]
                    if obj.get("results") is not None
                    else None
                ),
            }
        )
        return _obj



from datetime import datetime
from enum import Enum
from typing import Any, Dict, Iterable, List, Mapping, Optional, Union

from pydantic import Field, StrictBool, StrictInt, StrictStr
from typing_extensions import Annotated

from perigon.api_client import ApiClient
from perigon.models.company_search_result import CompanySearchResult
from perigon.models.journalist import Journalist
from perigon.models.journalist_search_result import JournalistSearchResult
from perigon.models.people_search_result import PeopleSearchResult
from perigon.models.sort_by import SortBy
from perigon.models.source_search_result import SourceSearchResult
from perigon.models.topic_search_result import TopicSearchResult

# Define API paths
PATH_GET_JOURNALIST_BY_ID = "/v1/journalists/{id}"
PATH_SEARCH_COMPANIES = "/v1/companies/all"
PATH_SEARCH_JOURNALISTS = "/v1/journalists/all"
PATH_SEARCH_PEOPLE = "/v1/people/all"
PATH_SEARCH_SOURCES = "/v1/sources/all"
PATH_SEARCH_TOPICS = "/v1/topics/all"


def _normalise_query(params: Mapping[str, Any]) -> Dict[str, Any]:
    """
    • Convert Enum → Enum.value
    • Convert list/tuple/set → CSV string (after Enum handling)
    • Skip None values
    """
    out: Dict[str, Any] = {}
    for key, value in params.items():
        if value is None:  # ignore "unset"
            continue

        # Unwrap single Enum
        if isinstance(value, Enum):  # Enum → str
            value = value.value

        # Handle datetime objects properly
        from datetime import datetime

        if isinstance(value, datetime):
            value = value.isoformat().split("+")[0]

        # Handle collection (after possible Enum unwrap)
        elif isinstance(value, (list, tuple, set)):
            # unwrap Enum members inside the collection
            items: Iterable[str] = (
                (
                    item.isoformat().replace(" ", "+")
                    if isinstance(item, datetime)
                    else str(item.value if isinstance(item, Enum) else item)
                )
                for item in value
            )
            value = ",".join(items)  # CSV join
        else:
            value = str(value)

        out[key] = value

    return out


class SupplementalEndpointsApi:
    """"""

    def __init__(self, api_client: Optional[ApiClient] = None):
        self.api_client = api_client or ApiClient()

    # ----------------- get_journalist_by_id (sync) ----------------- #
    def get_journalist_by_id(self, id: str) -> Journalist:
        """
        Find additional details on a journalist by using the journalist ID found in an article response object.

        Args:
            id (str): Parameter id (required)

        Returns:
            Journalist: The response
        """
        # Get path template from class attribute
        path = PATH_GET_JOURNALIST_BY_ID

        # Replace path parameters
        path = path.format(id=str(id))

        # --- build query dict on the fly ---
        params: Dict[str, Any] = {}
        params = _normalise_query(params)

        resp = self.api_client.request("GET", path, params=params)
        resp.raise_for_status()
        return Journalist.model_validate(resp.json())

    # ----------------- get_journalist_by_id (async) ----------------- #
    async def get_journalist_by_id_async(self, id: str) -> Journalist:
        """
        Async variant of get_journalist_by_id. Find additional details on a journalist by using the journalist ID found in an article response object.

        Args:
            id (str): Parameter id (required)

        Returns:
            Journalist: The response
        """
        # Get path template from class attribute
        path = PATH_GET_JOURNALIST_BY_ID

        # Replace path parameters
        path = path.format(id=str(id))

        params: Dict[str, Any] = {}
        params = _normalise_query(params)

        resp = await self.api_client.request_async("GET", path, params=params)
        resp.raise_for_status()
        return Journalist.model_validate(resp.json())

    # ----------------- search_companies (sync) ----------------- #
    def search_companies(
        self,
        id: Optional[List[str]] = None,
        symbol: Optional[List[str]] = None,
        domain: Optional[List[str]] = None,
        country: Optional[List[str]] = None,
        exchange: Optional[List[str]] = None,
        num_employees_from: Optional[int] = None,
        num_employees_to: Optional[int] = None,
        ipo_from: Optional[datetime] = None,
        ipo_to: Optional[datetime] = None,
        q: Optional[str] = None,
        name: Optional[str] = None,
        industry: Optional[str] = None,
        sector: Optional[str] = None,
        size: Optional[int] = None,
        page: Optional[int] = None,
    ) -> CompanySearchResult:
        """
        Browse or search for companies Perigon tracks using name, domain, ticker symbol, industry, and more. Supports Boolean search logic and filtering by metadata such as country, exchange, employee count, and IPO date.

        Args:
            id (Optional[List[str]]): Filter by unique company identifiers. Multiple values create an OR filter.
            symbol (Optional[List[str]]): Filter by company stock ticker symbols (e.g., AAPL, MSFT, GOOGL). Multiple values create an OR filter.
            domain (Optional[List[str]]): Filter by company domains or websites (e.g., apple.com, microsoft.com). Multiple values create an OR filter.
            country (Optional[List[str]]): Filter by company headquarters country. Multiple values create an OR filter.
            exchange (Optional[List[str]]): Filter by stock exchange where companies are listed (e.g., NASDAQ, NYSE). Multiple values create an OR filter.
            num_employees_from (Optional[int]): Filter for companies with at least this many employees.
            num_employees_to (Optional[int]): Filter for companies with no more than this many employees.
            ipo_from (Optional[datetime]): Filter for companies that went public on or after this date. Accepts ISO 8601 format (e.g., 2023-01-01T00:00:00) or yyyy-mm-dd format.
            ipo_to (Optional[datetime]): Filter for companies that went public on or before this date. Accepts ISO 8601 format (e.g., 2023-12-31T23:59:59) or yyyy-mm-dd format.
            q (Optional[str]): Primary search query for filtering companies across name, alternative names, domains, and ticker symbols. Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            name (Optional[str]): Search within company names. Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            industry (Optional[str]): Filter by company industry classifications. Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            sector (Optional[str]): Filter by company sector classifications. Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            size (Optional[int]): The number of companies to return per page in the paginated response.
            page (Optional[int]): The specific page of results to retrieve in the paginated response. Starts at 0.

        Returns:
            CompanySearchResult: The response
        """
        # Get path template from class attribute
        path = PATH_SEARCH_COMPANIES

        # --- build query dict on the fly ---
        params: Dict[str, Any] = {}
        if id is not None:
            params["id"] = id
        if symbol is not None:
            params["symbol"] = symbol
        if domain is not None:
            params["domain"] = domain
        if country is not None:
            params["country"] = country
        if exchange is not None:
            params["exchange"] = exchange
        if num_employees_from is not None:
            params["numEmployeesFrom"] = num_employees_from
        if num_employees_to is not None:
            params["numEmployeesTo"] = num_employees_to
        if ipo_from is not None:
            params["ipoFrom"] = ipo_from
        if ipo_to is not None:
            params["ipoTo"] = ipo_to
        if q is not None:
            params["q"] = q
        if name is not None:
            params["name"] = name
        if industry is not None:
            params["industry"] = industry
        if sector is not None:
            params["sector"] = sector
        if size is not None:
            params["size"] = size
        if page is not None:
            params["page"] = page
        params = _normalise_query(params)

        resp = self.api_client.request("GET", path, params=params)
        resp.raise_for_status()
        return CompanySearchResult.model_validate(resp.json())

    # ----------------- search_companies (async) ----------------- #
    async def search_companies_async(
        self,
        id: Optional[List[str]] = None,
        symbol: Optional[List[str]] = None,
        domain: Optional[List[str]] = None,
        country: Optional[List[str]] = None,
        exchange: Optional[List[str]] = None,
        num_employees_from: Optional[int] = None,
        num_employees_to: Optional[int] = None,
        ipo_from: Optional[datetime] = None,
        ipo_to: Optional[datetime] = None,
        q: Optional[str] = None,
        name: Optional[str] = None,
        industry: Optional[str] = None,
        sector: Optional[str] = None,
        size: Optional[int] = None,
        page: Optional[int] = None,
    ) -> CompanySearchResult:
        """
        Async variant of search_companies. Browse or search for companies Perigon tracks using name, domain, ticker symbol, industry, and more. Supports Boolean search logic and filtering by metadata such as country, exchange, employee count, and IPO date.

        Args:
            id (Optional[List[str]]): Filter by unique company identifiers. Multiple values create an OR filter.
            symbol (Optional[List[str]]): Filter by company stock ticker symbols (e.g., AAPL, MSFT, GOOGL). Multiple values create an OR filter.
            domain (Optional[List[str]]): Filter by company domains or websites (e.g., apple.com, microsoft.com). Multiple values create an OR filter.
            country (Optional[List[str]]): Filter by company headquarters country. Multiple values create an OR filter.
            exchange (Optional[List[str]]): Filter by stock exchange where companies are listed (e.g., NASDAQ, NYSE). Multiple values create an OR filter.
            num_employees_from (Optional[int]): Filter for companies with at least this many employees.
            num_employees_to (Optional[int]): Filter for companies with no more than this many employees.
            ipo_from (Optional[datetime]): Filter for companies that went public on or after this date. Accepts ISO 8601 format (e.g., 2023-01-01T00:00:00) or yyyy-mm-dd format.
            ipo_to (Optional[datetime]): Filter for companies that went public on or before this date. Accepts ISO 8601 format (e.g., 2023-12-31T23:59:59) or yyyy-mm-dd format.
            q (Optional[str]): Primary search query for filtering companies across name, alternative names, domains, and ticker symbols. Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            name (Optional[str]): Search within company names. Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            industry (Optional[str]): Filter by company industry classifications. Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            sector (Optional[str]): Filter by company sector classifications. Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            size (Optional[int]): The number of companies to return per page in the paginated response.
            page (Optional[int]): The specific page of results to retrieve in the paginated response. Starts at 0.

        Returns:
            CompanySearchResult: The response
        """
        # Get path template from class attribute
        path = PATH_SEARCH_COMPANIES

        params: Dict[str, Any] = {}
        if id is not None:
            params["id"] = id
        if symbol is not None:
            params["symbol"] = symbol
        if domain is not None:
            params["domain"] = domain
        if country is not None:
            params["country"] = country
        if exchange is not None:
            params["exchange"] = exchange
        if num_employees_from is not None:
            params["numEmployeesFrom"] = num_employees_from
        if num_employees_to is not None:
            params["numEmployeesTo"] = num_employees_to
        if ipo_from is not None:
            params["ipoFrom"] = ipo_from
        if ipo_to is not None:
            params["ipoTo"] = ipo_to
        if q is not None:
            params["q"] = q
        if name is not None:
            params["name"] = name
        if industry is not None:
            params["industry"] = industry
        if sector is not None:
            params["sector"] = sector
        if size is not None:
            params["size"] = size
        if page is not None:
            params["page"] = page
        params = _normalise_query(params)

        resp = await self.api_client.request_async("GET", path, params=params)
        resp.raise_for_status()
        return CompanySearchResult.model_validate(resp.json())

    # ----------------- search_journalists (sync) ----------------- #
    def search_journalists(
        self,
        id: Optional[List[str]] = None,
        q: Optional[str] = None,
        name: Optional[str] = None,
        twitter: Optional[str] = None,
        size: Optional[int] = None,
        page: Optional[int] = None,
        source: Optional[List[str]] = None,
        topic: Optional[List[str]] = None,
        category: Optional[List[str]] = None,
        label: Optional[List[str]] = None,
        min_monthly_posts: Optional[int] = None,
        max_monthly_posts: Optional[int] = None,
        country: Optional[List[str]] = None,
        updated_at_from: Optional[datetime] = None,
        updated_at_to: Optional[datetime] = None,
        show_num_results: Optional[bool] = None,
    ) -> JournalistSearchResult:
        """
        Search journalists using broad search attributes. Our database contains over 230,000 journalists from around the world and is refreshed frequently.

        Args:
            id (Optional[List[str]]): Filter by unique journalist identifiers. Multiple values create an OR filter to find journalists matching any of the specified IDs.
            q (Optional[str]): Primary search query for filtering journalists based on their name, title, and Twitter bio. Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            name (Optional[str]): Search specifically within journalist names. Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            twitter (Optional[str]): Filter journalists by their exact Twitter handle, without the @ symbol.
            size (Optional[int]): The number of journalists to return per page in the paginated response.
            page (Optional[int]): The specific page of results to retrieve in the paginated response. Starts at 0.
            source (Optional[List[str]]): Filter journalists by the publisher domains they write for. Supports wildcards (* and ?) for pattern matching (e.g., *.cnn.com). Multiple values create an OR filter.
            topic (Optional[List[str]]): Filter journalists by the topics they frequently cover. Multiple values create an OR filter to find journalists covering any of the specified topics.
            category (Optional[List[str]]): Filter journalists by the content categories they typically write about (e.g., Politics, Tech, Sports, Business). Multiple values create an OR filter.
            label (Optional[List[str]]): Filter journalists by the type of content they typically produce (e.g., Opinion, Paid-news, Non-news). Multiple values create an OR filter.
            min_monthly_posts (Optional[int]): Filter for journalists who publish at least this many articles per month. Used to identify more active journalists.
            max_monthly_posts (Optional[int]): Filter for journalists who publish no more than this many articles per month.
            country (Optional[List[str]]): Filter journalists by countries they commonly cover in their reporting. Uses ISO 3166-1 alpha-2 two-letter country codes in lowercase (e.g., us, gb, jp). Multiple values create an OR filter.
            updated_at_from (Optional[datetime]): Filter for journalist profiles updated on or after this date. Accepts ISO 8601 format (e.g., 2023-03-01T00:00:00) or yyyy-mm-dd format.
            updated_at_to (Optional[datetime]): Filter for journalist profiles updated on or before this date. Accepts ISO 8601 format (e.g., 2023-03-01T23:59:59) or yyyy-mm-dd format.
            show_num_results (Optional[bool]): Controls whether to return the exact result count. When false (default), counts are capped at 10,000 for performance reasons. Set to true for precise counts in smaller result sets.

        Returns:
            JournalistSearchResult: The response
        """
        # Get path template from class attribute
        path = PATH_SEARCH_JOURNALISTS

        # --- build query dict on the fly ---
        params: Dict[str, Any] = {}
        if id is not None:
            params["id"] = id
        if q is not None:
            params["q"] = q
        if name is not None:
            params["name"] = name
        if twitter is not None:
            params["twitter"] = twitter
        if size is not None:
            params["size"] = size
        if page is not None:
            params["page"] = page
        if source is not None:
            params["source"] = source
        if topic is not None:
            params["topic"] = topic
        if category is not None:
            params["category"] = category
        if label is not None:
            params["label"] = label
        if min_monthly_posts is not None:
            params["minMonthlyPosts"] = min_monthly_posts
        if max_monthly_posts is not None:
            params["maxMonthlyPosts"] = max_monthly_posts
        if country is not None:
            params["country"] = country
        if updated_at_from is not None:
            params["updatedAtFrom"] = updated_at_from
        if updated_at_to is not None:
            params["updatedAtTo"] = updated_at_to
        if show_num_results is not None:
            params["showNumResults"] = show_num_results
        params = _normalise_query(params)

        resp = self.api_client.request("GET", path, params=params)
        resp.raise_for_status()
        return JournalistSearchResult.model_validate(resp.json())

    # ----------------- search_journalists (async) ----------------- #
    async def search_journalists_async(
        self,
        id: Optional[List[str]] = None,
        q: Optional[str] = None,
        name: Optional[str] = None,
        twitter: Optional[str] = None,
        size: Optional[int] = None,
        page: Optional[int] = None,
        source: Optional[List[str]] = None,
        topic: Optional[List[str]] = None,
        category: Optional[List[str]] = None,
        label: Optional[List[str]] = None,
        min_monthly_posts: Optional[int] = None,
        max_monthly_posts: Optional[int] = None,
        country: Optional[List[str]] = None,
        updated_at_from: Optional[datetime] = None,
        updated_at_to: Optional[datetime] = None,
        show_num_results: Optional[bool] = None,
    ) -> JournalistSearchResult:
        """
        Async variant of search_journalists. Search journalists using broad search attributes. Our database contains over 230,000 journalists from around the world and is refreshed frequently.

        Args:
            id (Optional[List[str]]): Filter by unique journalist identifiers. Multiple values create an OR filter to find journalists matching any of the specified IDs.
            q (Optional[str]): Primary search query for filtering journalists based on their name, title, and Twitter bio. Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            name (Optional[str]): Search specifically within journalist names. Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            twitter (Optional[str]): Filter journalists by their exact Twitter handle, without the @ symbol.
            size (Optional[int]): The number of journalists to return per page in the paginated response.
            page (Optional[int]): The specific page of results to retrieve in the paginated response. Starts at 0.
            source (Optional[List[str]]): Filter journalists by the publisher domains they write for. Supports wildcards (* and ?) for pattern matching (e.g., *.cnn.com). Multiple values create an OR filter.
            topic (Optional[List[str]]): Filter journalists by the topics they frequently cover. Multiple values create an OR filter to find journalists covering any of the specified topics.
            category (Optional[List[str]]): Filter journalists by the content categories they typically write about (e.g., Politics, Tech, Sports, Business). Multiple values create an OR filter.
            label (Optional[List[str]]): Filter journalists by the type of content they typically produce (e.g., Opinion, Paid-news, Non-news). Multiple values create an OR filter.
            min_monthly_posts (Optional[int]): Filter for journalists who publish at least this many articles per month. Used to identify more active journalists.
            max_monthly_posts (Optional[int]): Filter for journalists who publish no more than this many articles per month.
            country (Optional[List[str]]): Filter journalists by countries they commonly cover in their reporting. Uses ISO 3166-1 alpha-2 two-letter country codes in lowercase (e.g., us, gb, jp). Multiple values create an OR filter.
            updated_at_from (Optional[datetime]): Filter for journalist profiles updated on or after this date. Accepts ISO 8601 format (e.g., 2023-03-01T00:00:00) or yyyy-mm-dd format.
            updated_at_to (Optional[datetime]): Filter for journalist profiles updated on or before this date. Accepts ISO 8601 format (e.g., 2023-03-01T23:59:59) or yyyy-mm-dd format.
            show_num_results (Optional[bool]): Controls whether to return the exact result count. When false (default), counts are capped at 10,000 for performance reasons. Set to true for precise counts in smaller result sets.

        Returns:
            JournalistSearchResult: The response
        """
        # Get path template from class attribute
        path = PATH_SEARCH_JOURNALISTS

        params: Dict[str, Any] = {}
        if id is not None:
            params["id"] = id
        if q is not None:
            params["q"] = q
        if name is not None:
            params["name"] = name
        if twitter is not None:
            params["twitter"] = twitter
        if size is not None:
            params["size"] = size
        if page is not None:
            params["page"] = page
        if source is not None:
            params["source"] = source
        if topic is not None:
            params["topic"] = topic
        if category is not None:
            params["category"] = category
        if label is not None:
            params["label"] = label
        if min_monthly_posts is not None:
            params["minMonthlyPosts"] = min_monthly_posts
        if max_monthly_posts is not None:
            params["maxMonthlyPosts"] = max_monthly_posts
        if country is not None:
            params["country"] = country
        if updated_at_from is not None:
            params["updatedAtFrom"] = updated_at_from
        if updated_at_to is not None:
            params["updatedAtTo"] = updated_at_to
        if show_num_results is not None:
            params["showNumResults"] = show_num_results
        params = _normalise_query(params)

        resp = await self.api_client.request_async("GET", path, params=params)
        resp.raise_for_status()
        return JournalistSearchResult.model_validate(resp.json())

    # ----------------- search_people (sync) ----------------- #
    def search_people(
        self,
        name: Optional[str] = None,
        wikidata_id: Optional[List[str]] = None,
        occupation_id: Optional[List[str]] = None,
        occupation_label: Optional[str] = None,
        page: Optional[int] = None,
        size: Optional[int] = None,
    ) -> PeopleSearchResult:
        """
        Search and retrieve additional information on known persons that exist within Perigon&#39;s entity database and as referenced in any article response object. Our database contains over 650,000 people from around the world and is refreshed frequently. People data is derived from Wikidata and includes a wikidataId field that can be used to lookup even more information on Wikidata&#39;s website.

        Args:
            name (Optional[str]): Search by person's name. Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            wikidata_id (Optional[List[str]]): Filter by Wikidata entity IDs (e.g., Q7747, Q937). These are unique identifiers from Wikidata.org that precisely identify public figures and eliminate name ambiguity. Multiple values create an OR filter.
            occupation_id (Optional[List[str]]): Filter by Wikidata occupation IDs (e.g., Q82955 for politician, Q33999 for actor, Q19546 for businessman). Finds people with specific professions. Multiple values create an OR filter.
            occupation_label (Optional[str]): Search by occupation name (e.g., politician, actor, CEO, athlete). Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            page (Optional[int]): The specific page of results to retrieve in the paginated response. Starts at 0.
            size (Optional[int]): The number of people to return per page in the paginated response.

        Returns:
            PeopleSearchResult: The response
        """
        # Get path template from class attribute
        path = PATH_SEARCH_PEOPLE

        # --- build query dict on the fly ---
        params: Dict[str, Any] = {}
        if name is not None:
            params["name"] = name
        if wikidata_id is not None:
            params["wikidataId"] = wikidata_id
        if occupation_id is not None:
            params["occupationId"] = occupation_id
        if occupation_label is not None:
            params["occupationLabel"] = occupation_label
        if page is not None:
            params["page"] = page
        if size is not None:
            params["size"] = size
        params = _normalise_query(params)

        resp = self.api_client.request("GET", path, params=params)
        resp.raise_for_status()
        return PeopleSearchResult.model_validate(resp.json())

    # ----------------- search_people (async) ----------------- #
    async def search_people_async(
        self,
        name: Optional[str] = None,
        wikidata_id: Optional[List[str]] = None,
        occupation_id: Optional[List[str]] = None,
        occupation_label: Optional[str] = None,
        page: Optional[int] = None,
        size: Optional[int] = None,
    ) -> PeopleSearchResult:
        """
        Async variant of search_people. Search and retrieve additional information on known persons that exist within Perigon&#39;s entity database and as referenced in any article response object. Our database contains over 650,000 people from around the world and is refreshed frequently. People data is derived from Wikidata and includes a wikidataId field that can be used to lookup even more information on Wikidata&#39;s website.

        Args:
            name (Optional[str]): Search by person's name. Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            wikidata_id (Optional[List[str]]): Filter by Wikidata entity IDs (e.g., Q7747, Q937). These are unique identifiers from Wikidata.org that precisely identify public figures and eliminate name ambiguity. Multiple values create an OR filter.
            occupation_id (Optional[List[str]]): Filter by Wikidata occupation IDs (e.g., Q82955 for politician, Q33999 for actor, Q19546 for businessman). Finds people with specific professions. Multiple values create an OR filter.
            occupation_label (Optional[str]): Search by occupation name (e.g., politician, actor, CEO, athlete). Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            page (Optional[int]): The specific page of results to retrieve in the paginated response. Starts at 0.
            size (Optional[int]): The number of people to return per page in the paginated response.

        Returns:
            PeopleSearchResult: The response
        """
        # Get path template from class attribute
        path = PATH_SEARCH_PEOPLE

        params: Dict[str, Any] = {}
        if name is not None:
            params["name"] = name
        if wikidata_id is not None:
            params["wikidataId"] = wikidata_id
        if occupation_id is not None:
            params["occupationId"] = occupation_id
        if occupation_label is not None:
            params["occupationLabel"] = occupation_label
        if page is not None:
            params["page"] = page
        if size is not None:
            params["size"] = size
        params = _normalise_query(params)

        resp = await self.api_client.request_async("GET", path, params=params)
        resp.raise_for_status()
        return PeopleSearchResult.model_validate(resp.json())

    # ----------------- search_sources (sync) ----------------- #
    def search_sources(
        self,
        domain: Optional[List[str]] = None,
        name: Optional[str] = None,
        source_group: Optional[str] = None,
        sort_by: Optional[SortBy] = None,
        page: Optional[int] = None,
        size: Optional[int] = None,
        min_monthly_visits: Optional[int] = None,
        max_monthly_visits: Optional[int] = None,
        min_monthly_posts: Optional[int] = None,
        max_monthly_posts: Optional[int] = None,
        country: Optional[List[str]] = None,
        source_country: Optional[List[str]] = None,
        source_state: Optional[List[str]] = None,
        source_county: Optional[List[str]] = None,
        source_city: Optional[List[str]] = None,
        source_lat: Optional[float] = None,
        source_lon: Optional[float] = None,
        source_max_distance: Optional[float] = None,
        category: Optional[List[str]] = None,
        topic: Optional[List[str]] = None,
        label: Optional[List[str]] = None,
        paywall: Optional[bool] = None,
        show_subdomains: Optional[bool] = None,
        show_num_results: Optional[bool] = None,
    ) -> SourceSearchResult:
        """
        Search and filter the 142,000+ media sources available via the Perigon API. The result includes a list of individual media sources that were matched to your specific criteria.

        Args:
            domain (Optional[List[str]]): Filter by specific publisher domains or subdomains. Supports wildcards (* and ?) for pattern matching (e.g., *.cnn.com, us?.nytimes.com). Multiple values create an OR filter.
            name (Optional[str]): Search by source name or alternative names. Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            source_group (Optional[str]): Filter by predefined publisher bundles (e.g., top100, top50tech). Returns all sources within the specified group. See documentation for available source groups.
            sort_by (Optional[SortBy]): Determines the source sorting order. Options include relevance (default, best match to query), globalRank (by overall traffic and popularity), monthlyVisits (by total monthly visitor count), and avgMonthlyPosts (by number of articles published monthly).
            page (Optional[int]): The specific page of results to retrieve in the paginated response. Starts at 0.
            size (Optional[int]): The number of sources to return per page in the paginated response.
            min_monthly_visits (Optional[int]): Filter for sources with at least this many monthly visitors. Used to target publishers by audience size.
            max_monthly_visits (Optional[int]): Filter for sources with no more than this many monthly visitors. Used to target publishers by audience size.
            min_monthly_posts (Optional[int]): Filter for sources that publish at least this many articles per month. Used to target publishers by content volume.
            max_monthly_posts (Optional[int]): Filter for sources that publish no more than this many articles per month. Used to target publishers by content volume.
            country (Optional[List[str]]): Filter sources by countries they commonly cover in their reporting. Uses ISO 3166-1 alpha-2 two-letter country codes in lowercase (e.g., us, gb, jp). See documentation for supported country codes. Multiple values create an OR filter.
            source_country (Optional[List[str]]): Filter for local publications based in specific countries. Uses ISO 3166-1 alpha-2 two-letter country codes in lowercase (e.g., us, gb, jp). See documentation for supported country codes. Multiple values create an OR filter.
            source_state (Optional[List[str]]): Filter for local publications based in specific states or regions. Uses standard two-letter state codes in lowercase (e.g., ca, ny, tx). See documentation for supported state codes. Multiple values create an OR filter.
            source_county (Optional[List[str]]): Filter for local publications based in specific counties. Multiple values create an OR filter.
            source_city (Optional[List[str]]): Filter for local publications based in specific cities. Multiple values create an OR filter.
            source_lat (Optional[float]): Latitude coordinate for filtering local publications by geographic proximity. Used with sourceLon and sourceMaxDistance for radius search.
            source_lon (Optional[float]): Longitude coordinate for filtering local publications by geographic proximity. Used with sourceLat and sourceMaxDistance for radius search.
            source_max_distance (Optional[float]): Maximum distance in kilometers from the coordinates defined by sourceLat and sourceLon. Defines the radius for local publication searches.
            category (Optional[List[str]]): Filter sources by their primary content categories such as Politics, Tech, Sports, Business, or Finance. Returns sources that frequently cover these topics. Multiple values create an OR filter.
            topic (Optional[List[str]]): Filter sources by their frequently covered topics (e.g., Markets, Cryptocurrency, Climate Change). Returns sources where the specified topic is among their top 10 covered areas. Multiple values create an OR filter.
            label (Optional[List[str]]): Filter sources by their content label patterns (e.g., Opinion, Paid-news, Non-news). Returns sources where the specified label is common in their published content. See documentation for all available labels. Multiple values create an OR filter.
            paywall (Optional[bool]): Filter by paywall status. Set to true to find sources with paywalls, or false to find sources without paywalls.
            show_subdomains (Optional[bool]): Controls whether subdomains are included as separate results. When true (default), subdomains appear as distinct sources. When false, results are consolidated to parent domains only.
            show_num_results (Optional[bool]): Controls whether to return the exact result count. When false (default), counts are capped at 10,000 for performance reasons. Set to true for precise counts in smaller result sets.

        Returns:
            SourceSearchResult: The response
        """
        # Get path template from class attribute
        path = PATH_SEARCH_SOURCES

        # --- build query dict on the fly ---
        params: Dict[str, Any] = {}
        if domain is not None:
            params["domain"] = domain
        if name is not None:
            params["name"] = name
        if source_group is not None:
            params["sourceGroup"] = source_group
        if sort_by is not None:
            params["sortBy"] = sort_by
        if page is not None:
            params["page"] = page
        if size is not None:
            params["size"] = size
        if min_monthly_visits is not None:
            params["minMonthlyVisits"] = min_monthly_visits
        if max_monthly_visits is not None:
            params["maxMonthlyVisits"] = max_monthly_visits
        if min_monthly_posts is not None:
            params["minMonthlyPosts"] = min_monthly_posts
        if max_monthly_posts is not None:
            params["maxMonthlyPosts"] = max_monthly_posts
        if country is not None:
            params["country"] = country
        if source_country is not None:
            params["sourceCountry"] = source_country
        if source_state is not None:
            params["sourceState"] = source_state
        if source_county is not None:
            params["sourceCounty"] = source_county
        if source_city is not None:
            params["sourceCity"] = source_city
        if source_lat is not None:
            params["sourceLat"] = source_lat
        if source_lon is not None:
            params["sourceLon"] = source_lon
        if source_max_distance is not None:
            params["sourceMaxDistance"] = source_max_distance
        if category is not None:
            params["category"] = category
        if topic is not None:
            params["topic"] = topic
        if label is not None:
            params["label"] = label
        if paywall is not None:
            params["paywall"] = paywall
        if show_subdomains is not None:
            params["showSubdomains"] = show_subdomains
        if show_num_results is not None:
            params["showNumResults"] = show_num_results
        params = _normalise_query(params)

        resp = self.api_client.request("GET", path, params=params)
        resp.raise_for_status()
        return SourceSearchResult.model_validate(resp.json())

    # ----------------- search_sources (async) ----------------- #
    async def search_sources_async(
        self,
        domain: Optional[List[str]] = None,
        name: Optional[str] = None,
        source_group: Optional[str] = None,
        sort_by: Optional[SortBy] = None,
        page: Optional[int] = None,
        size: Optional[int] = None,
        min_monthly_visits: Optional[int] = None,
        max_monthly_visits: Optional[int] = None,
        min_monthly_posts: Optional[int] = None,
        max_monthly_posts: Optional[int] = None,
        country: Optional[List[str]] = None,
        source_country: Optional[List[str]] = None,
        source_state: Optional[List[str]] = None,
        source_county: Optional[List[str]] = None,
        source_city: Optional[List[str]] = None,
        source_lat: Optional[float] = None,
        source_lon: Optional[float] = None,
        source_max_distance: Optional[float] = None,
        category: Optional[List[str]] = None,
        topic: Optional[List[str]] = None,
        label: Optional[List[str]] = None,
        paywall: Optional[bool] = None,
        show_subdomains: Optional[bool] = None,
        show_num_results: Optional[bool] = None,
    ) -> SourceSearchResult:
        """
        Async variant of search_sources. Search and filter the 142,000+ media sources available via the Perigon API. The result includes a list of individual media sources that were matched to your specific criteria.

        Args:
            domain (Optional[List[str]]): Filter by specific publisher domains or subdomains. Supports wildcards (* and ?) for pattern matching (e.g., *.cnn.com, us?.nytimes.com). Multiple values create an OR filter.
            name (Optional[str]): Search by source name or alternative names. Supports Boolean operators (AND, OR, NOT), exact phrases with quotes, and wildcards (* and ?) for flexible searching.
            source_group (Optional[str]): Filter by predefined publisher bundles (e.g., top100, top50tech). Returns all sources within the specified group. See documentation for available source groups.
            sort_by (Optional[SortBy]): Determines the source sorting order. Options include relevance (default, best match to query), globalRank (by overall traffic and popularity), monthlyVisits (by total monthly visitor count), and avgMonthlyPosts (by number of articles published monthly).
            page (Optional[int]): The specific page of results to retrieve in the paginated response. Starts at 0.
            size (Optional[int]): The number of sources to return per page in the paginated response.
            min_monthly_visits (Optional[int]): Filter for sources with at least this many monthly visitors. Used to target publishers by audience size.
            max_monthly_visits (Optional[int]): Filter for sources with no more than this many monthly visitors. Used to target publishers by audience size.
            min_monthly_posts (Optional[int]): Filter for sources that publish at least this many articles per month. Used to target publishers by content volume.
            max_monthly_posts (Optional[int]): Filter for sources that publish no more than this many articles per month. Used to target publishers by content volume.
            country (Optional[List[str]]): Filter sources by countries they commonly cover in their reporting. Uses ISO 3166-1 alpha-2 two-letter country codes in lowercase (e.g., us, gb, jp). See documentation for supported country codes. Multiple values create an OR filter.
            source_country (Optional[List[str]]): Filter for local publications based in specific countries. Uses ISO 3166-1 alpha-2 two-letter country codes in lowercase (e.g., us, gb, jp). See documentation for supported country codes. Multiple values create an OR filter.
            source_state (Optional[List[str]]): Filter for local publications based in specific states or regions. Uses standard two-letter state codes in lowercase (e.g., ca, ny, tx). See documentation for supported state codes. Multiple values create an OR filter.
            source_county (Optional[List[str]]): Filter for local publications based in specific counties. Multiple values create an OR filter.
            source_city (Optional[List[str]]): Filter for local publications based in specific cities. Multiple values create an OR filter.
            source_lat (Optional[float]): Latitude coordinate for filtering local publications by geographic proximity. Used with sourceLon and sourceMaxDistance for radius search.
            source_lon (Optional[float]): Longitude coordinate for filtering local publications by geographic proximity. Used with sourceLat and sourceMaxDistance for radius search.
            source_max_distance (Optional[float]): Maximum distance in kilometers from the coordinates defined by sourceLat and sourceLon. Defines the radius for local publication searches.
            category (Optional[List[str]]): Filter sources by their primary content categories such as Politics, Tech, Sports, Business, or Finance. Returns sources that frequently cover these topics. Multiple values create an OR filter.
            topic (Optional[List[str]]): Filter sources by their frequently covered topics (e.g., Markets, Cryptocurrency, Climate Change). Returns sources where the specified topic is among their top 10 covered areas. Multiple values create an OR filter.
            label (Optional[List[str]]): Filter sources by their content label patterns (e.g., Opinion, Paid-news, Non-news). Returns sources where the specified label is common in their published content. See documentation for all available labels. Multiple values create an OR filter.
            paywall (Optional[bool]): Filter by paywall status. Set to true to find sources with paywalls, or false to find sources without paywalls.
            show_subdomains (Optional[bool]): Controls whether subdomains are included as separate results. When true (default), subdomains appear as distinct sources. When false, results are consolidated to parent domains only.
            show_num_results (Optional[bool]): Controls whether to return the exact result count. When false (default), counts are capped at 10,000 for performance reasons. Set to true for precise counts in smaller result sets.

        Returns:
            SourceSearchResult: The response
        """
        # Get path template from class attribute
        path = PATH_SEARCH_SOURCES

        params: Dict[str, Any] = {}
        if domain is not None:
            params["domain"] = domain
        if name is not None:
            params["name"] = name
        if source_group is not None:
            params["sourceGroup"] = source_group
        if sort_by is not None:
            params["sortBy"] = sort_by
        if page is not None:
            params["page"] = page
        if size is not None:
            params["size"] = size
        if min_monthly_visits is not None:
            params["minMonthlyVisits"] = min_monthly_visits
        if max_monthly_visits is not None:
            params["maxMonthlyVisits"] = max_monthly_visits
        if min_monthly_posts is not None:
            params["minMonthlyPosts"] = min_monthly_posts
        if max_monthly_posts is not None:
            params["maxMonthlyPosts"] = max_monthly_posts
        if country is not None:
            params["country"] = country
        if source_country is not None:
            params["sourceCountry"] = source_country
        if source_state is not None:
            params["sourceState"] = source_state
        if source_county is not None:
            params["sourceCounty"] = source_county
        if source_city is not None:
            params["sourceCity"] = source_city
        if source_lat is not None:
            params["sourceLat"] = source_lat
        if source_lon is not None:
            params["sourceLon"] = source_lon
        if source_max_distance is not None:
            params["sourceMaxDistance"] = source_max_distance
        if category is not None:
            params["category"] = category
        if topic is not None:
            params["topic"] = topic
        if label is not None:
            params["label"] = label
        if paywall is not None:
            params["paywall"] = paywall
        if show_subdomains is not None:
            params["showSubdomains"] = show_subdomains
        if show_num_results is not None:
            params["showNumResults"] = show_num_results
        params = _normalise_query(params)

        resp = await self.api_client.request_async("GET", path, params=params)
        resp.raise_for_status()
        return SourceSearchResult.model_validate(resp.json())

    # ----------------- search_topics (sync) ----------------- #
    def search_topics(
        self,
        name: Optional[str] = None,
        category: Optional[str] = None,
        subcategory: Optional[str] = None,
        page: Optional[int] = None,
        size: Optional[int] = None,
    ) -> TopicSearchResult:
        """
        Search through all available Topics that exist within the Perigon Database.

        Args:
            name (Optional[str]): Search for topics by exact name or partial text match. Does not support wildcards. Examples include Markets, Cryptocurrency, Climate Change, etc.
            category (Optional[str]): Filter topics by broad article categories such as Politics, Tech, Sports, Business, Finance, Entertainment, etc.
            subcategory (Optional[str]): Filter topics by their specific subcategory. Subcategories provide more granular classification beyond the main category, such as TV or Event.
            page (Optional[int]): The specific page of results to retrieve in the paginated response. Starts at 0.
            size (Optional[int]): The number of topics to return per page in the paginated response.

        Returns:
            TopicSearchResult: The response
        """
        # Get path template from class attribute
        path = PATH_SEARCH_TOPICS

        # --- build query dict on the fly ---
        params: Dict[str, Any] = {}
        if name is not None:
            params["name"] = name
        if category is not None:
            params["category"] = category
        if subcategory is not None:
            params["subcategory"] = subcategory
        if page is not None:
            params["page"] = page
        if size is not None:
            params["size"] = size
        params = _normalise_query(params)

        resp = self.api_client.request("GET", path, params=params)
        resp.raise_for_status()
        return TopicSearchResult.model_validate(resp.json())

    # ----------------- search_topics (async) ----------------- #
    async def search_topics_async(
        self,
        name: Optional[str] = None,
        category: Optional[str] = None,
        subcategory: Optional[str] = None,
        page: Optional[int] = None,
        size: Optional[int] = None,
    ) -> TopicSearchResult:
        """
        Async variant of search_topics. Search through all available Topics that exist within the Perigon Database.

        Args:
            name (Optional[str]): Search for topics by exact name or partial text match. Does not support wildcards. Examples include Markets, Cryptocurrency, Climate Change, etc.
            category (Optional[str]): Filter topics by broad article categories such as Politics, Tech, Sports, Business, Finance, Entertainment, etc.
            subcategory (Optional[str]): Filter topics by their specific subcategory. Subcategories provide more granular classification beyond the main category, such as TV or Event.
            page (Optional[int]): The specific page of results to retrieve in the paginated response. Starts at 0.
            size (Optional[int]): The number of topics to return per page in the paginated response.

        Returns:
            TopicSearchResult: The response
        """
        # Get path template from class attribute
        path = PATH_SEARCH_TOPICS

        params: Dict[str, Any] = {}
        if name is not None:
            params["name"] = name
        if category is not None:
            params["category"] = category
        if subcategory is not None:
            params["subcategory"] = subcategory
        if page is not None:
            params["page"] = page
        if size is not None:
            params["size"] = size
        params = _normalise_query(params)

        resp = await self.api_client.request_async("GET", path, params=params)
        resp.raise_for_status()
        return TopicSearchResult.model_validate(resp.json())



# Package : perigon
from __future__ import annotations

from typing import Any, Dict, Optional

import httpx


class ApiClient:
    """
    Single entry‑point that wraps an httpx.Client *and* httpx.AsyncClient.
    Accepts api_key & base_url directly instead of a separate Configuration.
    """

    def __init__(
        self,
        api_key: Optional[str] = None,
        base_url: str = "https://api.perigon.io",
        timeout: Optional[float] = None,
    ):
        self.api_key = api_key
        self.base_url = base_url or "https://api.perigon.io"
        self.timeout = timeout

        # Persistent sessions for connection‑pool reuse (HTTP/1.1 or HTTP/2)
        self._sync = httpx.Client(base_url=self.base_url, timeout=self.timeout)
        self._async = httpx.AsyncClient(base_url=self.base_url, timeout=self.timeout)

    # ------------------------------------------------------------------ #
    # Internal helpers
    # ------------------------------------------------------------------ #
    def _auth_headers(self) -> Dict[str, str]:
        hdrs: Dict[str, str] = {}

        if self.api_key:
            hdrs["Authorization"] = f"Bearer {self.api_key}"

        return hdrs

    def _prepare_url(self, path: str) -> str:
        url = f"{self.base_url}{path}"
        return url

    # ------------------------------------------------------------------ #
    # Public request wrappers
    # ------------------------------------------------------------------ #
    def request(
        self,
        method: str,
        path: str,
        headers: Optional[Dict[str, str]] = None,
        **kwargs: Any,
    ) -> httpx.Response:
        all_headers = self._auth_headers()
        if headers:  # merge user‑provided headers
            all_headers.update(headers)
        url = self._prepare_url(path)
        return self._sync.request(method, url, headers=all_headers, **kwargs)

    async def request_async(
        self,
        method: str,
        path: str,
        headers: Optional[Dict[str, str]] = None,
        **kwargs: Any,
    ) -> httpx.Response:
        all_headers = self._auth_headers()
        if headers:
            all_headers.update(headers)
        url = self._prepare_url(path)
        return await self._async.request(method, url, headers=all_headers, **kwargs)

    # ------------------------------------------------------------------ #
    # Clean‑up helpers
    # ------------------------------------------------------------------ #
    def close(self) -> None:
        self._sync.close()

    async def aclose(self) -> None:
        await self._async.aclose()
